#!/usr/bin/python3
import os
import subprocess
from concurrent.futures import ThreadPoolExecutor


def main():
    meme_file = "H12CORE_meme_format.meme"
    genome_file = "GRCh38_no_alt_analysis_set_GCA_000001405.15.fasta"

    # Read the MEME file
    with open(meme_file, "r") as f:
        lines = f.readlines()

    # Extract header (everything before the first MOTIF line)
    header_lines = []
    motif_blocks = []
    current_motif = []
    in_motif = False
    for line in lines:
        if line.startswith("MOTIF"):
            if in_motif:
                # Save the previous motif
                motif_blocks.append(current_motif)
                current_motif = [line]
            else:
                # First motif encountered
                in_motif = True
                current_motif = [line]
        elif in_motif:
            current_motif.append(line)
        else:
            header_lines.append(line)
    # Add the last motif
    if in_motif and current_motif:
        motif_blocks.append(current_motif)

    # Write individual motif files
    motif_files = []
    for motif_block in motif_blocks:
        # Extract motif name from the MOTIF line
        motif_line = motif_block[0]
        motif_name = motif_line.strip().split()[1]
        if not os.path.exists("motifs"):
            os.makedirs("motifs")
        motif_filename = f"motifs/{motif_name}.meme"
        with open(motif_filename, "w") as f:
            f.writelines(header_lines)
            f.writelines(motif_block)
        motif_files.append(motif_filename)

    # Function to run fimo on a motif file
    def run_fimo(motif_file):
        output_dir = f'fimo_out/{os.path.basename(motif_file).replace(".meme", "")}'
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
        cmd = ["fimo", "--oc", output_dir, "--max-stored-scores", "1000000000", motif_file, genome_file]
        print(f"Running fimo on {motif_file}...")
        result = subprocess.run(cmd, capture_output=True, text=True)
        if result.returncode != 0:
            print(f"Error running fimo on {motif_file}:\n{result.stderr}")
        else:
            print(f"fimo completed on {motif_file}")

    # Run fimo in parallel using ThreadPoolExecutor
    with ThreadPoolExecutor(max_workers=100) as executor:
        executor.map(run_fimo, motif_files)


if __name__ == "__main__":
    main()
